// $Id: hoftMon.hh,v 1.14 2009/03/31 00:52:30 jordi Exp $

#ifndef HOFTMON_HH
#define HOFTMON_HH

#include "DVecType.hh"
#include "DatEnv.hh"
#include "FrWriter.hh"
#include "MonServer.hh"
#include "FixedLenTS.hh"
#include "Trend.hh"
#include <string>

extern "C" {
#include <lal/Calibration.h>
}


/*
 * @name    hoftMon
 * @memo    Compute calibrated h(t)
 * @author  Jordi Burguet-Castell, Shourov K. Chatterji, Xavier Siemens
 * @version 0.98
 */
class hoftMon : public DatEnv, MonServer {
public:
    hoftMon(int argc, const char *argv[]);
    ~hoftMon(void);
    void ProcessData(void);      // process 1 data stride
    void Attention(void);        // handle message signal

private:
    double    mStride;         // time stride (duration of the data chunk)
    FrWriter  mFrameWriter;    // frame writer
    int       mIteration;      // in which iteration we are
    int       mStepsSinceLastDrop;  // how many steps since last dropout
    INT4      mLastTime;       // when we read the last stride (in s)

    //  Phase hack...
    long mNPhase;
    long mPhaseNo;

    /* LALComputeStrain() data structures */
    StrainIn  mInputData;
    StrainOut mOutputData;

    /* Take care of the last bad times seen in the previous blocks */
    int m_t_bad_left_1, m_t_bad_left_2, m_t_bad_left_3;
    
    /* Output channels */
    TSeries mLastSV;   // state vector from one iteration ago
    TSeries mLastDQ;

    /* Extra necessary channels */
    TSeries mLastLAX;  // normalized light in x-arm from one iteration ago
    TSeries mLastLAY;  // normalized light in y-arm from one iteration ago

    /* Input channel names */
    static std::string mCalChanNames[];  // calibration (see below)
    std::string mSVInChanName;  // state vector (x:IFO-SV_STATE_VECTOR)
    std::string mLAXChanName;   // light-in-arms X (x:LSC-LA_PTRX_NORM)
    std::string mLAYChanName;   // light-in-arms Y (x:LSC-LA_PTRY_NORM)

    /* Ouput channel names */
    std::string mSVChanName;  // state vector (x:DMT-STATE_VECTOR)
    std::string mDQChanName;  // data quality vector (x:DMT-DATA_QUALITY_VECTOR)
    std::string mStrainChanName;  // calibrated h(t) (x:DMT-STRAIN)

    std::string mOutputFileName;  // name of the output file (hoft-%s-16.gwf)

    int  get_cal_channels(const TSeries* dmt_cal_chans[]);
    int  initialize_input_data(int argc, const char *argv[]);
    int  read_config_file(const char* cfile);
    int  allocate_data(const TSeries* dmt_cal_chans[]);
    int  free_data();
    int  destroy_initialization();
    void read_cal_data(const TSeries* dmt_cal_chans[]);
    int  write_frames(const TSeries* dmt_cal_chans[]);
    void set_channel_names(std::string ifo);
    TSeries compute_dq(int t_bad_left, int t_bad_right,
                       TSeries& SV, TSeries& LAX, TSeries& LAY);
};


// Default names of the calibration channels we will read.
std::string hoftMon::mCalChanNames[] = {
  "H2:LSC-AS_Q",
  "H2:LSC-DARM_CTRL",
  "H2:LSC-DARM_ERR",
  "H2:LSC-DARM_CTRL_EXC_DAQ"
};


/*
 * From
 * https://www.lsc-group.phys.uwm.edu/daswg/wiki/S6OnlineGroup/CalibratedData
 *
 * State vector channel: IFO:DMT-STATE_VECTOR
 *
 * Bits 0-4 define the states. bits 5-15 are always set to 1, such
 * that the word 0xffff means science mode:
 *
 * bit0=SCI (operator set to go to science mode)
 * bit1=CON conlog unsets this bit is non-harmless epics changes
 * bit2=UP (set by locking scripts)
 * bit3=!INJ Injections unset this bit
 * bit4=EXC Unauthorized excitations cause this bit to be unset 
 *
 * Channel Name: IFO:DMT-DATA_QUALITY_VECTOR where IFO is one of (H1, H2, L1)
 * Sample Rate: 1 Hz, for any state to be true, it must be true every
 * sample of h(t) within the second
 * Type: INT4
 *
 * The quality channel will use the following bitmask definition
 *
 * SCIENCE       1  // SV_SCIENCE & LIGHT
 * INJECTION     2  // Injection: same as statevector 
 * UP            4  // SV_UP & LIGHT
 * CALIBRATED    8  // SV_UP & LIGHT & (not TRANSIENT)
 * BADGAMMA     16  // Calibration is bad (outside 0.8 < gamma < 1.2)
 * LIGHT        32  // Light in the arms ok
 * MISSING      64  // Indication that data was dropped in DMT
 */


#endif     //  HOFTMON_HH
