/* -*- mode: c++; c-basic-offset: 4; -*- */
#ifndef TRIG_SEG_ACCOUNTANT_HH
#define TRIG_SEG_ACCOUNTANT_HH

#include "Time.hh"
#include "Interval.hh"
#include <string>
#include <vector>
#include <iostream>

class TrigClient;

namespace trig {

    class Segment;

    /**  The SegAccountant class performs segment accounting for a single
      *  segment generation process. Specifically, it:
      *  <ul>
      *    <li>Allows user to define multiple segments.</li>
      *    <li>Tracks segment status and whether it is defined.</li>
      *    <li>Floshes segment according to one of three different modes</li>
      *    <li> Allows user to define multiple segments.</li>>
      *  </ul>
      *
      *  Usage:
      *
      *  Initialization:
      *
      *  \verbatim
      *  SegAccountant sa(tc); // Construct
      *
      *  sa.addSegment("SegName", 1, kAligned, 16.0);
      *  \endverbatim
      *
      *  Stride processing;
      *
      *  \verbatim
      *  sa.start_stride(ts.getStartTime());
      *  ...
      *  
      *  ...
      *  sa.update(ts.getEndTime());
      *  \endverbatim
      *
      *  \version 1.0; Last modified Jan 5, 2009
      *  \author  John G. zweizig
      */
    class SegAccountant {
    public:
	/** Enumerate the segment flush modes.
	 */
	enum flush_mode {
	    kNone,    // Allow segments to accumulate until the segment 
	              // changes state or until the accountant is deleted.
	    kMaxTime, // impose a maximum time period on all segments.
	    kAligned, // Flush at regular boundaries.
	    kWriteThrough // write each segment immediately.
	};

	/** Segment identifier.
	  */
	class seg_id {
	public:
	    /**  Construct and empty segment id.
	      */
	    seg_id(void);

	    /**  Construct a segment information entry for a named segment.
	      *  Online segments should have version 1.
	      *  @memo construct a segment information entry.
	      */
	    seg_id(const std::string& name, int version=1);

	    /** Destructor
	     */
	    virtual ~seg_id(void);

	    /**  Get the ifo name.
	     */
	    std::string full_name(void) const;

	    /**  Get the ifo name.
	     */
	    const char* ifo(void) const;

	    /**  Get the segment name.
	     */
	    const char* name(void) const;

	    /**  Set the ifo.
	     */
	    void setIfo(const std::string& ifo);

	    /**  Get the segment number.
	     */
	    int version(void) const;

	    /**  Test for equal eegment id
	     */
	    bool operator==(const seg_id& si) const;
	    bool operator==(const std::string& si) const;
	    bool operator!=(const seg_id& si) const;
	    bool operator!=(const std::string& si) const;

	    /** Test for segid less.
	     */
	    bool operator<(const seg_id& si) const;

	    /**  Return a constant reference to the Ifo name;
	     */
	    const std::string& refIfo(void) const;

	    /**  Return a constant reference to the segment name;
	     */
	    const std::string& refName(void) const;

	private:
	    std::string _segname;
	    std::string _ifo;
	    int         _version;
	};

	/**  Segment information structure.
	  */
	class seg_info : public seg_id {
	public:
	    /**  Construct a segment information entry for a named segment.
	      *  Online segments should have version 1.
	      *  @memo construct a segment information entry.
	      */
	    seg_info(const seg_id& sid, const std::string& comment, 
		     Interval maxt=0.0);

	    /**  Destroy a segment information entry.
	      *  Online segments should have version 1.
	      */
	    ~seg_info(void);

	    /**  Get the last update time.
	     */
	    const Time& last(void) const;

	    /**  Write the current segment to the trigger client. If the 
	      *  end parameter is not null (Time(0)), the segment up to 
	      *  the specified end time is writeen and that portion is
	      *  deleted from the current segment.
	      */
	    void seg_write(TrigClient* tc, const Time& end=Time(0));

	    /**  Set the segment state
	     */
	    void set_segment(TrigClient* tc, const Time& start, 
			     const Time& end, bool onoff);

	    /**  Get the segment start time.
	     */
	    const Time& start(void) const;

	    /**  Get the segment state.
	     */
	    bool test(void) const;

	    /**  Write out the current segment.
	     */
	    void update(TrigClient* tc, const Time& end=Time(0));

	private:
	    //--------------------------  Segment parameters
	    std::string _comment;
	    Interval    _maxtime;

	    //--------------------------  Segment status.
	    Time        _start;
	    Time        _last;
	    bool        _state;
	};

    public:
	/**  Construct an unattached segment accountant.
	  *  \brief Default constructor.
	  */
	SegAccountant(void);

	/**  Construct an unattached segment accountant.
	  *  \brief Default constructor.
	  */
	SegAccountant(TrigClient& tc);

	/**  Destroy an unattached segment accountant.
	  *  \brief Default constructor.
	  */
	~SegAccountant(void);

	/**  Add a segment to be accounted. The name is 
	 */
	void addSegment(const std::string& name, int version=1, 
			const std::string& comment="", Interval max_time=0.0);

	/**  Add a segment to be accounted. The name is 
	 */
	void addSegment(const seg_id& name, const std::string& comment="", 
			Interval max_time=0.0);

	/**  Close and delete the trigger client.
	  *  \brief Close the segment accountant.
	  */
	void close(void);

	/**  Dump out the contents.
	  */
	std::ostream& dump(std::ostream& out) const;

	/**  Find the specified segment, return a reference. If no segment
	  *  is found matching the specified ID, an exception is thrown.
	  */
	seg_info& find(const seg_id& sid);

	/**  Return a string describing the mode.
	  */
	static const char* flush_mode_to_char(flush_mode fm);

	/**  Return a mode name by the string.
	  */
	static flush_mode flush_char_to_mode(const std::string& fc);

	/**  Find the specified segment, return an index into the list of
	  *  the specified segment or the index at which the segment would
	  *  be inserted.
	  */
	int locate(const seg_id& sid);

	/**  Set the segment flush mode and timing.
	  */
	void set_mode(flush_mode m, Interval t=0);

	/**  Set segment state in specified time interval.
	  */
	void set_segment(const std::string& si, const Time& start, 
			 const Time& end, bool onoff);

	/**  Set segment state in specified time interval.
	  */
	void set_segment(const seg_id& si, const Time& start, 
			 const Time& end, bool onoff);

	/**  set up at start of analysis segment.
	  */
	void start_stride(const Time& ts);

	/**  Get the time for the nex file to be written
	  */
	Time test_write(const Time& tup) const;

	/**  Update to the specified time.
	  */
	void update(const Time& tup);

    private:
	typedef std::vector<seg_info>    seg_list;
	typedef seg_list::iterator       seg_iter;
	typedef seg_list::const_iterator const_seg_iter;

    private:
	seg_list    mList;
	TrigClient* mTC;
	flush_mode  mFlush;
	Interval    mFlushInterval;
	Time        mStart;
	Time        mLastUpdate;
	Time        mLatest;
    };

    //==================================  SegAccountant::seg_info inlines
    inline const char*
    SegAccountant::seg_id::ifo(void) const {
	return _ifo.c_str();
    }

    inline const char*
    SegAccountant::seg_id::name(void) const {
	return _segname.c_str();
    }

    inline const std::string& 
    SegAccountant::seg_id::refIfo(void) const {
	return _ifo;
    }

    inline const std::string& 
    SegAccountant::seg_id::refName(void) const {
	return _segname;
    }

    inline int 
    SegAccountant::seg_id::version(void) const {
	return _version;
    }

    inline bool 
    SegAccountant::seg_id::operator==(const seg_id& si) const {
	return _segname == si._segname && _version == si._version;
    }

    inline bool 
    SegAccountant::seg_id::operator!=(const seg_id& si) const {
	return ! operator==(si);
    }

    inline bool 
    SegAccountant::seg_id::operator==(const std::string& si) const {
	return full_name() == si;
    }
 
    inline bool 
    SegAccountant::seg_id::operator!=(const std::string& si) const {
	return ! operator==(si);
    }

    inline std::string
    SegAccountant::seg_id::full_name(void) const {
	if (_ifo.empty()) return _segname;
	return _ifo + ":" + _segname;
    }

    inline bool 
    SegAccountant::seg_id::operator<(const seg_id& si) const {
	if (_ifo  < si._ifo) return true;
	if (_ifo != si._ifo) return false;
	if (_segname  < si._segname) return true;
	if (_segname != si._segname) return false;
	if (_version  < si._version) return true;
	return false;
    }

    inline std::ostream&
    operator<<(std::ostream& out, const SegAccountant::seg_id& si) {
	return out << si.refIfo() << ":" << si.name() << ":" << si.version();
    }

    inline const Time&
    SegAccountant::seg_info::last(void) const {
	return _last;
    }

    inline const Time& 
    SegAccountant::seg_info::start(void) const {
	return _start;
    }

    inline bool 
    SegAccountant::seg_info::test(void) const {
	return _state;
    }
}

#endif  // !defined(TRIG_SEG_ACCOUNTANT_HH)
