
//
//  Class FDFilter: General frequency-domain filter class.
//    version:  0.3 
//       date:  2004.09.24   
//    authors:  Patrick J. Sutton (psutton@ligo.caltech.edu)
//              John G. Zweizig (jzweizig@ligo.caltech.edu)
//
///////////////////////////////////////////////////////////////////////////


#ifndef FDFilter_HH
#define FDFilter_HH

#include "FDFilterBase.hh"

/**  The FDFilter class applies a frequency domain filter specified as an 
  *  FSpectrum to data in the same format; ie, it
  *  multiplies FSpectrum objects frequency-by-frequency.
  */
class FDFilter : public FDFilterBase {

public:
   using FDFilterBase::Apply;

  /**  Default constructor.
    */
  FDFilter(void);

  /**  Constructor using a filter specified as an FSpectrum. 
    */
  FDFilter(const FSpectrum& transfunc);

  /**  Constructor using a filter specified as an FSeries. 
    */
  FDFilter(const FSeries& transfunc);

  /**  Destructor
    */
  virtual ~FDFilter(void);

  /**  Filter the specified FSpectrum. 
    *  This method extracts from the specified input FSpectrum that frequency
    *  range for which the filter is defined, multiplies it by the filter 
    *  function, and writes the result to the specified output FSpectrum.
    *  The input FSpectrum must be sampled with the same frequency spacing 
    *  as the filter. Apply must work correctly when the input Spectrum is
    *  the same as the output spectrum i.e. when filtering is 
    *  performed in place.
    *  @param Int_PSD Input FSpectrum to be filtered.
    *  @param Out_PSD Filtered output FSpectrum.
    */
  virtual void Apply(const FSpectrum& Int_PSD, FSpectrum& Out_PSD);

  /**  Filter the specified FSeries. 
    *  This method extracts from the specified FSeries that frequency
    *  range for which the filter is defined, multiplies it by the filter 
    *  function, and returns. Apply must work correctly when the input 
    *  series is the same as the output series i.e. when filtering is 
    *  performed in place.
    */
  virtual void Apply(const FSeries& Int_PSD, FSeries& Out_PSD);

  /**  This method extracts from the specified FSpectrum that frequency
    *  range for which the filter is defined, multiplies it by the filter 
    *  function, and returns the filtered FSpectrum.
    *  The input FSpectrum must be sampled with the same frequency spacing 
    *  as the filter.
    *  @memo Filter the specified FSpectrum. 
    *  @param Int_PSD Input FSpectrum to be filtered.
    */
  virtual FSpectrum Apply(const FSpectrum& Int_PSD);

  /**  This method extracts from the specified FSpectrum that frequency
    *  range for which the filter is defined, multiplies it by the filter 
    *  function, and returns.
    *  @memo Filter the specified FSeries.
    */
  virtual FSeries Apply(const FSeries& Int_DFT);

  /**  Create an identical copy of this filter.
    *  @memo Clone the filter.
    */
  virtual FDFilter* clone(void) const;

  /**  Retrieve the filter into an FSpectrum.
    */
  const FSpectrum& getFSpectrum(void) const;

  /**  Return the maximum frequency for which the filter is defined.
    *  @memo Return maximum frequency for which the filter is defined.
    */ 
  double getHighFreq(void) const;

  /**  Return the minimum frequency for which the filter is defined.
    *  @memo Return minimum frequency for which the filter is defined.
    */
  double getLowFreq(void) const;

  /**  Define the filter for a new FSpectrum.
    */
  void setFDFilter(const FSpectrum& calib);

private:

  //  FSpectrum holding filter function.
  FSpectrum FSpecFilter;

  //  FSpectrum holding filter function.
  FSeries   FSeriesFilter;

  //  Is filter stored as FSeries, so phase data is available (true if yes)?
  bool isFSeries;

};

//======================================  Inline methods
inline const FSpectrum& 
FDFilter::getFSpectrum(void) const
{
  // cout << "FDFilter::getFSpectrum called.\n";
  return FSpecFilter;
}

inline double 
FDFilter::getHighFreq(void) const
{
  // cout << "FDFilter::getHighFreq called.\n";
  return FSpecFilter.getHighFreq();
}

inline double
FDFilter::getLowFreq(void) const
{
  // cout << "FDFilter::getLowFreq called.\n";
  return FSpecFilter.getLowFreq();
}

#endif  //----- FDFilter_HH
